package cn.iocoder.yudao.module.system.service.information;

import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.module.grpc.api.GrpcServiceApi;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.information.vo.DeviceInformationSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.information.DeviceInformationDO;
import cn.iocoder.yudao.module.system.dal.mysql.information.DeviceInformationMapper;
import cn.iocoder.yudao.module.system.enums.device.DeviceInformationEnum;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;


import java.time.LocalDateTime;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 设备信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
@Slf4j
public class DeviceInformationServiceImpl implements DeviceInformationService {

    @Resource
    private DeviceInformationMapper informationMapper;

    @Resource
    private GrpcServiceApi grpcServiceApi;

    @Override
    public Long createInformation(DeviceInformationSaveReqVO createReqVO) {
        DeviceInformationPageReqVO pageReqVO = new DeviceInformationPageReqVO();
        pageReqVO.setMacAddress(createReqVO.getMacAddress());
        PageResult<DeviceInformationDO> pageResult = informationMapper.selectPage(pageReqVO);
        if(ObjectUtil.isNotEmpty(pageResult.getList())){
            throw exception(INFORMATION_MAC_EXIST);
        }
        // 插入
        DeviceInformationDO information = BeanUtils.toBean(createReqVO, DeviceInformationDO.class);
        information.setDeviceStatus(1);
        information.setDeviceLastTime(LocalDateTime.now());
        informationMapper.insert(information);
        // 返回
        return information.getId();
    }

    @Override
    public void updateInformation(DeviceInformationSaveReqVO updateReqVO) {
        // 校验存在
        validateInformationExists(updateReqVO.getId());
        // 更新
        DeviceInformationDO updateObj = BeanUtils.toBean(updateReqVO, DeviceInformationDO.class);
        informationMapper.updateById(updateObj);
    }

    @Override
    public void deleteInformation(Long id) {
        // 校验存在
        validateInformationExists(id);
        // 删除
        informationMapper.deleteById(id);
    }

    private void validateInformationExists(Long id) {
        if (informationMapper.selectById(id) == null) {
            throw exception(INFORMATION_NOT_EXISTS);
        }
    }

    @Override
    public DeviceInformationDO getInformation(Long id) {
        return informationMapper.selectById(id);
    }

    @Override
    public PageResult<DeviceInformationDO> getInformationPage(DeviceInformationPageReqVO pageReqVO) {
        return informationMapper.selectPage(pageReqVO);
    }

}