package cn.iocoder.yudao.module.system.service.robot;

import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotWarnMsgPageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.robot.vo.RobotWarnMsgSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.robot.RobotWarnMsgDO;
import cn.iocoder.yudao.module.system.dal.mysql.robot.RobotWarnMsgMapper;
import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.WARN_MSG_NOT_EXISTS;

/**
 * 机器人告警信息 Service 实现类
 *
 * @author 陈宾顺
 */
@Service
@Validated
public class RobotWarnMsgServiceImpl implements RobotWarnMsgService {

    @Resource
    private RobotWarnMsgMapper warnMsgMapper;

    @Override
    public Long createWarnMsg(RobotWarnMsgSaveReqVO createReqVO) {
        // 插入
        RobotWarnMsgDO warnMsg = BeanUtils.toBean(createReqVO, RobotWarnMsgDO.class);
        warnMsgMapper.insert(warnMsg);
        // 返回
        return warnMsg.getId();
    }

    @Override
    public void updateWarnMsg(RobotWarnMsgSaveReqVO updateReqVO) {
        // 校验存在
        validateWarnMsgExists(updateReqVO.getId());
        // 更新
        RobotWarnMsgDO updateObj = BeanUtils.toBean(updateReqVO, RobotWarnMsgDO.class);
        warnMsgMapper.updateById(updateObj);
    }

    @Override
    public void deleteWarnMsg(Long id) {
        // 校验存在
        validateWarnMsgExists(id);
        // 删除
        warnMsgMapper.deleteById(id);
    }

    private void validateWarnMsgExists(Long id) {
        if (warnMsgMapper.selectById(id) == null) {
            throw exception(WARN_MSG_NOT_EXISTS);
        }
    }

    @Override
    public RobotWarnMsgDO getWarnMsg(Long id) {
        return warnMsgMapper.selectById(id);
    }

    @Override
    public PageResult<RobotWarnMsgDO> getWarnMsgPage(RobotWarnMsgPageReqVO pageReqVO) {
        return warnMsgMapper.selectPage(pageReqVO);
    }

}